﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using EAP.Data;
using EAP;
using EAP.Data.Validation;
using EAP.Entity;
using EAP.Collections;

namespace Demo
{
    public class AccountGroupManager : IAccountGroupManager
    {
        public AccountGroupModels GetModels(string name)
        {
            AccountGroupTable table = new AccountGroupTable();
            SelectStatement sql = DataAccess.DefaultDB
                .Select(table, table.AllColumns())//读取所有字段
                .OrderBy(table.UpdateDate.Desc);//按修改时间倒序
            if (!name.IsNullOrEmpty())//如果名称不为空，添加模糊查询
                sql.Where(table.Name.Like("%" + name + "%"));
            using (SafeDataReader sdr = sql.ToSafeDataReader())
            {
                AccountGroupModels models = new AccountGroupModels();
                while (sdr.Read())
                {
                    AccountGroupModel m = new AccountGroupModel();
                    m.BeginInit();//把model的状态设为Initializing，修改值时不会触发什么事件
                    sdr.Fetch(m, table);//把数据加载到model
                    m.EndInit();
                    models.Add(m);
                }
                return models;
            }
        }

        public void Save(List<AccountGroupModel> models, string user)
        {
            Validator v = new Validator();//构建验证器
            using (DbTransaction tran = DataAccess.DefaultDB.BeginTransaction())//开始事务
            {
                foreach (var m in models)
                {
                    CheckModel(v, m, tran);//验证数据
                    m.UpdateUser = user;
                    m.UpdateDate = System.DateTime.Now;
                    if (m.DataState == DataState.Created)
                        AddModel(m, tran);
                    else if (m.DataState == DataState.Modified)
                        UpdateModel(m, tran);
                    else if (m.DataState == DataState.Deleted)
                        DeleteModel(m, tran);
                }
                if (!v.IsValid)//验证不通过时抛出异常
                    throw new ValidationException(v);
                tran.Commit();
            }
        }

        protected int AddModel(AccountGroupModel model, DbTransaction tran)
        {
            AccountGroupTable table = new AccountGroupTable();
            return DataAccess.DefaultDB.Insert(table)
                .SetTransaction(tran)
                .AddColumns(model, table.Id)
                .ExecuteIdentity(table.Id);
        }

        protected int UpdateModel(AccountGroupModel model, DbTransaction tran)
        {
            AccountGroupTable table = new AccountGroupTable();
            return DataAccess.DefaultDB.Update(table)
                .SetTransaction(tran)
                .AddColumns(model, table.Id)
                .Where(table.Id == model.Id)
                .Execute();
        }

        protected int DeleteModel(AccountGroupModel model, DbTransaction tran)
        {
            AccountGroupTable table = new AccountGroupTable();
            return DataAccess.DefaultDB.Delete(table)
                .SetTransaction(tran)
                .Where(table.Id == model.Id)
                .Execute();
        }

        protected bool Exists(AccountGroupModel model, DbTransaction tran)
        {
            AccountGroupTable table = new AccountGroupTable();
            SelectStatement sql = DataAccess.DefaultDB
                .Select(table, table.AllColumns(false).Count())
                .SetTransaction(tran)
                .Where(table.Name == model.Name);
            if (model.DataState != EAP.Entity.DataState.Created)
                sql.Where(table.Id != model.Id);
            return sql.ToScalar<int>() > 0;
        }

        protected void CheckModel(Validator v, AccountGroupModel model, DbTransaction tran)
        {
            //名称不能为空，长度不能超过64字符
            if (v.Require(model.Name, "Name", model.DisplayIndex)
                && v.MaxLength(model.Name, 64, "Name", model.DisplayIndex))
            {
                //名称不能重复
                v.Assert(!Exists(model, tran), "Name", model.DisplayIndex, ErrorText.Exists);
            }
            v.MaxLength(model.UpdateUser, 32, "UpdateUser", model.DisplayIndex);
        }

        public ValueTextList GetGroupList()
        {
            AccountGroupTable table = new AccountGroupTable();
            SelectStatement sql = DataAccess.DefaultDB
                .Select(table, table.Id, table.Name)
                .Where(table.Visible == true)
                .OrderBy(table.Name.Asc);
            using (SafeDataReader sdr = sql.ToSafeDataReader())
            {
                ValueTextList list = new ValueTextList();
                while (sdr.Read())
                {
                    ValueTextPair m = new ValueTextPair();
                    m.Value = sdr.GetInt32(table.Id);
                    m.Text = sdr.GetString(table.Name);
                    list.Add(m);
                }
                return list;
            }
        }
    }
}
